/*
 * file: listener.c
 * purpose: listen on a pipe for input from the active file system.
 *          when input is received, act on it.
 *
 * 29-Apr EvanG: created
 */

#include <stdio.h>
#include <fcntl.h>

#include "/usr/src/linux/fs/active/active.h"

static char sLogFile[] = "/usr/src/linux/fs/active/active.log";

/* read characters from the pipe until receive an EOL */
static int readln(int fd, char *buf) {
    int len1 ;
    char c = '\0';
    int i = 0;

    while (c != '\n') {
        len1 = read(fd, &c, 1);
        buf[i] = c;
        i++;
    }
    buf[i] = '\0';

    return i;
}

main()
{
    int flags, mode ;
    int fd_k2l, fd_l2k, fd_log ;
    int pid ;
    char ack[] = "Y";
    char buf[MAX_AUDIT_BUFSIZE];
    int len ;

    /* create a named pipe to listen for the kernel */
    flags = O_RDWR ;
    mode = O_RDWR ;
    fd_k2l = open("/usr/src/linux/fs/active/listener/np_k2l", flags, mode);
    if (fd_k2l < 0) {
        printf("Listener: Error: could not open a named pipe.\n");
        exit(1);
    }

    /* Should the listener send an acknowledgement to the kernel? */
    if (DO_ACKNOWLEDGEMENT) {
        /* create a named pipe for the listener to talk to the kernel */
        flags = O_RDWR | O_NONBLOCK ;
        mode = O_RDWR | O_NONBLOCK ;
        fd_l2k = open("/usr/src/linux/fs/active/listener/np_l2k", flags, mode);
        if (fd_l2k < 0) {
            printf("Listener: Error: could not open a named pipe.\n");
            exit(1);
        }
    }

    /* run and listen forever */
    while (1) {
//printf("Listener parent process is about to call readln to read.\n");
        /* get a line of input from the kernel */
        len = readln(fd_k2l, &buf[0]);
//printf("Listener heard (%d chars): %s", len, buf);

        /* write the audit information to the log */
        /* Open the file, creating it if necessary */
        flags = O_CREAT | O_WRONLY | O_APPEND ;
        mode = 0666;
        fd_log = open(sLogFile, flags, mode);
        if (fd_log < 0) {
            printf("Listener: Error: could not open the log file.\n");
            exit(1);
        } else {
            /* write to the audit file */
            len = write(fd_log, buf, len);
            close(fd_log);
        }

        /* is an acknowledgement wanted? */
        if (DO_ACKNOWLEDGEMENT) {
            /* acknowledge the message */
//printf("Listener tells the talker: %s.\n", ack);
            if (len > 0) {
                len = write(fd_l2k, &ack[0], 1);
            }
        }
    }    /* end while */

    close(fd_k2l);
    if (DO_ACKNOWLEDGEMENT) {
        close(fd_l2k);
    }

    exit (0);
}
